package hardcaml_event_driven_sim

  1. Overview
  2. Docs

Module Simulator.Change_monitorSource

Sourcetype t
Sourceval create : Signal_id.t list -> t

Setup a monitor for waiting on a given list of signals.

Sourceval wait : t -> unit Async.Deferred.t

Wait until any of the signals changes a value.