package hardcaml_event_driven_sim

  1. Overview
  2. Docs

Module type Simulator.Value_SSource

type t
val sexp_of_t : t -> Sexplib0.Sexp.t
val (=) : t -> t -> bool
val resolve_value : [ `Unresolved | `Func of last_value:t -> t list -> t ]

What to do if there are multiple processes driving the signal?

`Unresolved - throw an exception `Func - invoke a function with a list of value to determine the result

val check_value_compatibility : t -> unit

Checks if t is a correct value for this signal type.

val initial_value : t

What value should this signal have before any processes starts driving it?